%% LCGT-SAS model
N = 100;
cd('/home/aso/Desktop/aso/exp/LCGT/2010/HeatLink/Suspension');

seismicModel = 3; % 1: Measured data, 2: 1/f, 3: 1/f^2


% Heatlink is anchored to PF
g=9.8;

%Prepare seismic data
if seismicModel == 1
    %Seismic raw data
    n=160;  
    fp=fopen('kamioka1.txt','r');
    [Data,count]=fscanf(fp,'%f',[2,n]);
    fclose(fp);
    I=[1:n];
    x = 10.^(I/40-4+log10(20));
    y = Data(2,:);
    %Interpolation of data
    N=10000;
    freq = linspace(20/N,20,N);
    w = 2*pi*freq;
    seismic=interp1(x,y,freq).*0.00005./w;
elseif seismicModel == 2    
    freq = logspace(-1,2,N);
    seismic = 5e-10./(freq.^1);
else
    freq = logspace(-1,2,N);
    seismic = 1.5e-9./(freq.^2);
end

% Filter0
w0=sqrt(g/50); %50
m0=120; %120
Q0=1;
k0=m0*w0^2;

% Filter1
w1=sqrt(g/0.8);
m1=m0;
Q1=100;
k1=m1*w1^2;

% Filter2
w2=sqrt(g/0.8);
m2=m0;
Q2=100;
k2=m2*w2^2;

% PF
w3=sqrt(g/1);
m3=m0;
Q3=100;
k3=m3*w3^2;

% MB
w4=sqrt(g/0.5);
m4=60;
Q4=1000;
k4=m4*w4^2;

% IM
w5=sqrt(g/0.5);
m5=60;
Q5=1000;
k5=m5*w5^2;

C=530; % 530

% TM
w6=sqrt(g/0.4);
m6=30;
Q6=1000;
k6=m6*w6^2;

% RM
w7=sqrt(g/0.4);
m7=30;
Q7=6; %6
k7=m7*w7^2;

% Heat Link
w8=2*pi*0.0142; %0.03
m8=m3;
Q8=420; %420
k8=m8*w8^2;

% Inertial damping
G0=10; %10
f0=0.1; %0.2 -> 0.1 center frequency
fp=0.05; %0.03 -> 0.05
Qp=0.5; %0.5
fb=0.17; %0.17 boost frequency
Qb=10; %10 boost Q

%% Load Heatlink TF X

data = dlmread('/home/aso/Desktop/aso/exp/LCGT/2010/HeatLink/HeatLink7/Data/TFXreal.dat');
fhl = data(:,1);
TFr = data(:,2);

data = dlmread('/home/aso/Desktop/aso/exp/LCGT/2010/HeatLink/HeatLink7/Data/TFXimag.dat');
fhl = data(:,1);
TFi = data(:,2);

TF = -(TFr +i*TFi);
%Interpolation
%TFr = interp1(fhl, TFr,freq);
%TFi = interp1(fhl, TFi,freq);
%TF = interp1(fhl, TF, freq, 'nearest');

whl = 2*pi*fhl;
%KHL
KHL = (whl.^2 .* m3 .* TF)./(TF-1);


%% Equation of motion for total
for I=1:numel(fhl)
    w=2*pi*fhl(I);
    G=G0*(1+fp/(1i*fhl(I))/Qp+(fp/(1i*fhl(I)))^2)/(1+1i*fhl(I)/f0)/(1+f0/(1i*fhl(I)));
    G=G*(1+(1i*fhl(I))/fb/Qp+((1i*fhl(I))/fb)^2)/(1+(1i*fhl(I))/fb/Qb+((1i*fhl(I))/fb)^2); %Boost
    M=[
    -(1+G)*m0*w^2 + K(m0,k0,w,Q0) + K(m1,k1,w,Q1), -K(m1,k1,w,Q1), 0, 0, 0, 0, 0, 0
    -K(m1,k1,w,Q1), -m1*w^2 + K(m1,k1,w,Q1) + K(m2,k2,w,Q2), -K(m2,k2,w,Q2), 0, 0, 0, 0, 0
    0, -K(m2,k2,w,Q2), -m2*w^2 + K(m2,k2,w,Q2) + K(m3,k3,w,Q3), -K(m3,k3,w,Q3), 0, 0, 0, 0
    0, 0, -K(m3,k3,w,Q3), -m3*w^2 + K(m3,k3,w,Q3) + K(m4,k4,w,Q4) + K(m5,k5,w,Q5) + KHL(I), -K(m4,k4,w,Q4), -K(m5,k5,w,Q5), 0, 0
    0, 0, 0, -K(m4,k4,w,Q4), -m4*w^2 + K(m4,k4,w,Q4) + C*1i*w, -C*1i*w, 0, 0
    0, 0, 0, -K(m5,k5,w,Q5), -C*1i*w, -m5*w^2 + K(m5,k5,w,Q5) + K(m6,k6,w,Q6) + K(m7,k7,w,Q7) + C*1i*w, -K(m6,k6,w,Q6), -K(m7,k7,w,Q7)
    0, 0, 0, 0, 0, -K(m6,k6,w,Q6), -m6*w^2 + K(m6,k6,w,Q6), 0
    0, 0, 0, 0, 0, -K(m7,k7,w,Q7), 0, -m7*w^2 + K(m7,k7,w,Q7)
    ];
    Xg=[K(m0,k0,w,Q0);0;0;KHL(I);0;0;0;0];
    X=M \ Xg;
    X6(I)=X(7);
    %Gnd(I)=seismic(I);
    %Disp(I)=seismic(I)*abs(X(7));
end

%interpolate
TFabs = interp1(fhl, abs(X6), freq);
TFangle = interp1(fhl, angle(X6), freq);
TF = TFabs.*exp(i*TFangle);


%% Export total

M = [freq(5:end)', real(TF(5:end))', imag(TF(5:end))', (TFabs(5:end).*seismic(5:end))', seismic(5:end)'];
dlmwrite('data/TotalX.dat',M);

%% Equation of motion for HL only

for I=1:numel(fhl)
    w=2*pi*fhl(I);
    G=G0*(1+fp/(1i*fhl(I))/Qp+(fp/(1i*fhl(I)))^2)/(1+1i*fhl(I)/f0)/(1+f0/(1i*fhl(I)));
    G=G*(1+(1i*fhl(I))/fb/Qp+((1i*fhl(I))/fb)^2)/(1+(1i*fhl(I))/fb/Qb+((1i*fhl(I))/fb)^2); %Boost
    M=[
    -(1+G)*m0*w^2 + K(m0,k0,w,Q0) + K(m1,k1,w,Q1), -K(m1,k1,w,Q1), 0, 0, 0, 0, 0, 0
    -K(m1,k1,w,Q1), -m1*w^2 + K(m1,k1,w,Q1) + K(m2,k2,w,Q2), -K(m2,k2,w,Q2), 0, 0, 0, 0, 0
    0, -K(m2,k2,w,Q2), -m2*w^2 + K(m2,k2,w,Q2) + K(m3,k3,w,Q3), -K(m3,k3,w,Q3), 0, 0, 0, 0
    0, 0, -K(m3,k3,w,Q3), -m3*w^2 + K(m3,k3,w,Q3) + K(m4,k4,w,Q4) + K(m5,k5,w,Q5) + KHL(I), -K(m4,k4,w,Q4), -K(m5,k5,w,Q5), 0, 0
    0, 0, 0, -K(m4,k4,w,Q4), -m4*w^2 + K(m4,k4,w,Q4) + C*1i*w, -C*1i*w, 0, 0
    0, 0, 0, -K(m5,k5,w,Q5), -C*1i*w, -m5*w^2 + K(m5,k5,w,Q5) + K(m6,k6,w,Q6) + K(m7,k7,w,Q7) + C*1i*w, -K(m6,k6,w,Q6), -K(m7,k7,w,Q7)
    0, 0, 0, 0, 0, -K(m6,k6,w,Q6), -m6*w^2 + K(m6,k6,w,Q6), 0
    0, 0, 0, 0, 0, -K(m7,k7,w,Q7), 0, -m7*w^2 + K(m7,k7,w,Q7)
    ];
    Xg=[0;0;0;KHL(I);0;0;0;0];
    X=M \ Xg;
    X6(I)=X(7);
    %Gnd(I)=seismic(I);
    %Disp(I)=seismic(I)*abs(X(7));
end

%interpolate
TFabs = interp1(fhl, abs(X6), freq);
TFangle = interp1(fhl, angle(X6), freq);
TF = TFabs.*exp(i*TFangle);


%% Export HL only
M = [freq(5:end)', real(TF(5:end))', imag(TF(5:end))', (TFabs(5:end).*seismic(5:end))', seismic(5:end)'];
dlmwrite('data/HLonlyX.dat',M);

%% Export Model

M = [freq(5:end)', real(TF(5:end))', imag(TF(5:end))', (TFabs(5:end).*seismic(5:end))', seismic(5:end)'];
dlmwrite('data/TotalXModel2.dat',M);
%% Load Heatlink TF Y

data = dlmread('/home/aso/Desktop/aso/exp/LCGT/2010/HeatLink/HeatLink7/Data/TFYreal.dat');
fhl = data(:,1);
TFr = data(:,2);

data = dlmread('/home/aso/Desktop/aso/exp/LCGT/2010/HeatLink/HeatLink7/Data/TFYimag.dat');
fhl = data(:,1);
TFi = data(:,2);

TF = -(TFr +i*TFi);
%Interpolation
%TFr = interp1(fhl, TFr,freq);
%TFi = interp1(fhl, TFi,freq);
%TF = interp1(fhl, TF, freq, 'nearest');

whl = 2*pi*fhl;
%KHL
KHL = (whl.^2 .* m3 .* TF)./(TF-1);

%% Equation of motion for total
for I=1:numel(fhl)
    w=2*pi*fhl(I);
    G=G0*(1+fp/(1i*fhl(I))/Qp+(fp/(1i*fhl(I)))^2)/(1+1i*fhl(I)/f0)/(1+f0/(1i*fhl(I)));
    G=G*(1+(1i*fhl(I))/fb/Qp+((1i*fhl(I))/fb)^2)/(1+(1i*fhl(I))/fb/Qb+((1i*fhl(I))/fb)^2); %Boost
    M=[
    -(1+G)*m0*w^2 + K(m0,k0,w,Q0) + K(m1,k1,w,Q1), -K(m1,k1,w,Q1), 0, 0, 0, 0, 0, 0
    -K(m1,k1,w,Q1), -m1*w^2 + K(m1,k1,w,Q1) + K(m2,k2,w,Q2), -K(m2,k2,w,Q2), 0, 0, 0, 0, 0
    0, -K(m2,k2,w,Q2), -m2*w^2 + K(m2,k2,w,Q2) + K(m3,k3,w,Q3), -K(m3,k3,w,Q3), 0, 0, 0, 0
    0, 0, -K(m3,k3,w,Q3), -m3*w^2 + K(m3,k3,w,Q3) + K(m4,k4,w,Q4) + K(m5,k5,w,Q5) + KHL(I), -K(m4,k4,w,Q4), -K(m5,k5,w,Q5), 0, 0
    0, 0, 0, -K(m4,k4,w,Q4), -m4*w^2 + K(m4,k4,w,Q4) + C*1i*w, -C*1i*w, 0, 0
    0, 0, 0, -K(m5,k5,w,Q5), -C*1i*w, -m5*w^2 + K(m5,k5,w,Q5) + K(m6,k6,w,Q6) + K(m7,k7,w,Q7) + C*1i*w, -K(m6,k6,w,Q6), -K(m7,k7,w,Q7)
    0, 0, 0, 0, 0, -K(m6,k6,w,Q6), -m6*w^2 + K(m6,k6,w,Q6), 0
    0, 0, 0, 0, 0, -K(m7,k7,w,Q7), 0, -m7*w^2 + K(m7,k7,w,Q7)
    ];
    Xg=[K(m0,k0,w,Q0);0;0;KHL(I);0;0;0;0];
    X=M \ Xg;
    X6(I)=X(7);
    %Gnd(I)=seismic(I);
    %Disp(I)=seismic(I)*abs(X(7));
end

%interpolate
TFabs = interp1(fhl, abs(X6), freq);
TFangle = interp1(fhl, angle(X6), freq);
TF = TFabs.*exp(i*TFangle);



%% Export total

M = [freq(5:end)', real(TF(5:end))', imag(TF(5:end))', (TFabs(5:end).*seismic(5:end))', seismic(5:end)'];
dlmwrite('data/TotalY.dat',M);

%% Equation of motion for HL only
for I=1:numel(fhl)
    w=2*pi*fhl(I);
    G=G0*(1+fp/(1i*fhl(I))/Qp+(fp/(1i*fhl(I)))^2)/(1+1i*fhl(I)/f0)/(1+f0/(1i*fhl(I)));
    G=G*(1+(1i*fhl(I))/fb/Qp+((1i*fhl(I))/fb)^2)/(1+(1i*fhl(I))/fb/Qb+((1i*fhl(I))/fb)^2); %Boost
    M=[
    -(1+G)*m0*w^2 + K(m0,k0,w,Q0) + K(m1,k1,w,Q1), -K(m1,k1,w,Q1), 0, 0, 0, 0, 0, 0
    -K(m1,k1,w,Q1), -m1*w^2 + K(m1,k1,w,Q1) + K(m2,k2,w,Q2), -K(m2,k2,w,Q2), 0, 0, 0, 0, 0
    0, -K(m2,k2,w,Q2), -m2*w^2 + K(m2,k2,w,Q2) + K(m3,k3,w,Q3), -K(m3,k3,w,Q3), 0, 0, 0, 0
    0, 0, -K(m3,k3,w,Q3), -m3*w^2 + K(m3,k3,w,Q3) + K(m4,k4,w,Q4) + K(m5,k5,w,Q5) + KHL(I), -K(m4,k4,w,Q4), -K(m5,k5,w,Q5), 0, 0
    0, 0, 0, -K(m4,k4,w,Q4), -m4*w^2 + K(m4,k4,w,Q4) + C*1i*w, -C*1i*w, 0, 0
    0, 0, 0, -K(m5,k5,w,Q5), -C*1i*w, -m5*w^2 + K(m5,k5,w,Q5) + K(m6,k6,w,Q6) + K(m7,k7,w,Q7) + C*1i*w, -K(m6,k6,w,Q6), -K(m7,k7,w,Q7)
    0, 0, 0, 0, 0, -K(m6,k6,w,Q6), -m6*w^2 + K(m6,k6,w,Q6), 0
    0, 0, 0, 0, 0, -K(m7,k7,w,Q7), 0, -m7*w^2 + K(m7,k7,w,Q7)
    ];
    Xg=[0;0;0;KHL(I);0;0;0;0];
    X=M \ Xg;
    X6(I)=X(7);
    %Gnd(I)=seismic(I);
    %Disp(I)=seismic(I)*abs(X(7));
end

%interpolate
TFabs = interp1(fhl, abs(X6), freq);
TFangle = interp1(fhl, angle(X6), freq);
TF = TFabs.*exp(i*TFangle);


%% Export HL only
M = [freq(5:end)', real(TF(5:end))', imag(TF(5:end))', (TFabs(5:end).*seismic(5:end))', seismic(5:end)'];
dlmwrite('data/HLonlyY.dat',M);

%% Export Model

M = [freq(5:end)', real(TF(5:end))', imag(TF(5:end))', (TFabs(5:end).*seismic(5:end))', seismic(5:end)'];
dlmwrite('data/TotalYModel2.dat',M);

%% Plot
figure(1);
subplot(3,1,[1 2]), loglog(freq,abs(X6)), xlabel('Frequency [Hz]'), ylabel('Amplitude'), grid on, grid minor,
 axis([0.01, 20, 1e-8, 10]), set(gca,'ytick',[1e-8 1e-7 1e-6 1e-5 1e-4 1e-3 1e-2 1e-1 1 10]);
 legend('X6/Xg');
 hold on
subplot(3,1,3), semilogx(freq,angle(X6)*180/pi),
 xlabel('Frequency [Hz]'), ylabel('Phase [deg]'), grid on, grid minor, axis([0.01, 20, -180, 180]),
 set(gca,'ytick',[-180 -120 -60 0 60 120 180]);
 hold on
figure(2);
 loglog(freq,Gnd,freq,Disp,'r'), xlabel('Frequency [Hz]'), ylabel('Displacement [m/rtHz]'),
 grid on, grid minor, axis([0.01, 20, 1e-22, 0.0001]), set(gca,'ytick',[1e-22 1e-20 1e-18 1e-16 1e-14 1e-12 1e-10 1e-8 1e-6 1e-4]);
 legend('Ground','Testmass');
hold on
%% Save to file 
%fp=fopen('disp.txt','w');
%fprintf(fp,'%e\n',Disp);
%fclose(fp);
save('HL1.mat','freq','Gnd','Disp');
